==> rheolef_6.7.orig.tar.gz: rheolef-6.7/doc/pexamples/p_laplacian_post.cc <==
#include "rheolef.h"
using namespace rheolef;
using namespace std;
#include "eta.icc"
void usage(string name) {
  derr << name << ": usage: "
       << name << " "
       << "[-|file[.field[.gz]]] "
       << "[-check] "
       << "[-residue] "
       << "[-criteria] "
       << endl;
  exit (1);
}
field residue (Float p, const field& uh) {
  const space& Xh = uh.get_space();
  trial u (Xh); test v (Xh);
  field lh = integrate (v);
  form  m  = integrate (u*v);
  quadrature_option_type qopt;
  qopt.set_family (quadrature_option_type::gauss);
  qopt.set_order  (2*Xh.degree()-1);
  form a  = integrate (compose (eta(p), norm2(grad(uh)))*dot(grad(u),grad(v)), qopt);
  field mrh = a*uh - lh;
  mrh["boundary"] = 0;
  field rh (Xh);
  solver sm = ldlt(m.uu());
  rh.set_u() = sm.solve(mrh.u());
  rh["boundary"] = 0;
  return rh;
}
void check (Float p, const field& uh) {
  field rh = residue(p, uh);
  const space& Xh = rh.get_space();
  trial u (Xh); test v (Xh);
  form m = integrate (u*v);
  Float res = sqrt(m(rh,rh));
  derr << "check: residue = " << res << endl;
  check_macro (res < 1e-3, "unexpected residue");
}
field criteria (Float p, const field& uh) {
  const space& Xh = uh.get_space();
  if (uh.get_approx() == "P1") return interpolate (Xh, abs(uh));
  return interpolate (Xh, pow(norm(grad(uh)), p/2));
}
void
load (idiststream& in, Float& p, field& uh) {
    in >> catchmark("p")     >> p
       >> catchmark("u")     >> uh;
}
int main(int argc, char**argv) {
  environment rheolef (argc,argv);
  if (argc < 3) usage(argv[0]);
  Float p;
  field uh;
  if (string (argv[1]) == "-") {
    load (din, p, uh);
  } else {
    idiststream in;
    in.open(argv[1], "field");
    load (in, p, uh);
  }
  for (int i = 2; i < argc; i++) {
         if (strcmp (argv[i], "-check") == 0) { check (p, uh); }
    else if (strcmp (argv[i], "-residue") == 0) { dout << catchmark("ru")   << residue (p, uh); }
    else if (strcmp (argv[i], "-criteria") == 0) { dout << catchmark("c")   << criteria (p, uh); }
    else {
       derr << argv[0] << ": unknown option \'" << argv[i] << "'" << endl;
       usage(argv[0]);
    }
  }
}

==> rheolef_7.0.orig.tar.gz: rheolef-7.0/doc/pexamples/p_laplacian_post.cc <==
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2018 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================
#include "rheolef.h"
using namespace rheolef;
using namespace std;
#include "eta.icc"
void usage(string name) {
  derr << name << ": usage: "
       << name << " "
       << "[-|file[.field[.gz]]] "
       << "[-check] "
       << "[-residue] "
       << "[-criteria] "
       << endl;
  exit (1);
}
field residue (Float p, const field& uh) {
  const space& Xh = uh.get_space();
  trial u (Xh); test v (Xh);
  field lh = integrate (v);
  form  m  = integrate (u*v);
  quadrature_option qopt;
  qopt.set_family (quadrature_option::gauss);
  qopt.set_order  (2*Xh.degree()-1);
  form a  = integrate (compose (eta(p), norm2(grad(uh)))*dot(grad(u),grad(v)), qopt);
  field mrh = a*uh - lh;
  mrh["boundary"] = 0;
  field rh (Xh);
  solver sm = ldlt(m.uu());
  rh.set_u() = sm.solve(mrh.u());
  rh["boundary"] = 0;
  return rh;
}
void check (Float p, const field& uh) {
  field rh = residue(p, uh);
  const space& Xh = rh.get_space();
  trial u (Xh); test v (Xh);
  form m = integrate (u*v);
  Float res = sqrt(m(rh,rh));
  derr << "check: residue = " << res << endl;
  check_macro (res < 1e-3, "unexpected residue");
}
field criteria (Float p, const field& uh) {
  const space& Xh = uh.get_space();
  if (uh.get_approx() == "P1") return interpolate (Xh, abs(uh));
  return interpolate (Xh, pow(norm(grad(uh)), p/2));
}
void
load (idiststream& in, Float& p, field& uh) {
    in >> catchmark("p")     >> p
       >> catchmark("u")     >> uh;
}
int main(int argc, char**argv) {
  environment rheolef (argc,argv);
  if (argc < 3) usage(argv[0]);
  Float p;
  field uh;
  if (string (argv[1]) == "-") {
    load (din, p, uh);
  } else {
    idiststream in;
    in.open(argv[1], "field");
    load (in, p, uh);
  }
  for (int i = 2; i < argc; i++) {
         if (strcmp (argv[i], "-check") == 0) { check (p, uh); }
    else if (strcmp (argv[i], "-residue") == 0) { dout << catchmark("ru")   << residue (p, uh); }
    else if (strcmp (argv[i], "-criteria") == 0) { dout << catchmark("c")   << criteria (p, uh); }
    else {
       derr << argv[0] << ": unknown option \'" << argv[i] << "'" << endl;
       usage(argv[0]);
    }
  }
}
